@@ -9,10 +9,10 @@ package Lemonldap::NG::Common::Conf;
 
 use strict;
 no strict 'refs';
-use Lemonldap::NG::Common::Conf::Constants;    #inherits
+use Lemonldap::NG::Common::Conf::Constants;     #inherits
+use Lemonldap::NG::Common::Conf::Attributes;    #inherits
 use Lemonldap::NG::Common::Crypto
   ;    #link protected cipher Object "cypher" in configuration hash
-use Regexp::Assemble;
 use Config::IniFiles;
 
 #inherits Lemonldap::NG::Common::Conf::File
@@ -20,7 +20,7 @@ use Config::IniFiles;
 #inherits Lemonldap::NG::Common::Conf::SOAP
 #inherits Lemonldap::NG::Common::Conf::LDAP
 
-our $VERSION = '1.2.4';
+our $VERSION = '1.4.0';
 our $msg;
 our $iniObj;
 
@@ -115,15 +115,14 @@ sub saveConf {
         return DATABASE_LOCKED if ( $self->isLocked() or not $self->lock() );
     }
     $conf->{cfgNum} = $last + 1 unless ( $self->{cfgNumFixed} );
-    foreach my $k (qw(reVHosts cipher)) {
-        delete( $conf->{$k} );
-    }
+    delete $conf->{cipher};
 
     # Try to store configuration
     my $tmp = $self->store($conf);
 
     unless ( $tmp > 0 ) {
         $msg .= "Configuration $conf->{cfgNum} not stored.\n";
+        $self->unlock();
         return ( $tmp ? $tmp : UNKNOWN_ERROR );
     }
 
@@ -141,18 +140,22 @@ sub saveConf {
 # @return Lemonldap::NG configuration
 sub getConf {
     my ( $self, $args ) = @_;
-    if (    $args->{'local'}
+
+    # Use only cache to get conf
+    if (    $args->{local}
         and ref( $self->{refLocalStorage} )
         and my $res = $self->{refLocalStorage}->get('conf') )
     {
         $msg .= "Get configuration from cache without verification.\n";
         return $res;
     }
+
+    # Check cfgNum in conf backend
+    # Get conf in backend only if a newer configuration is available
     else {
         $args->{cfgNum} ||= $self->lastCfg;
         unless ( $args->{cfgNum} ) {
-            $msg .= "No configuration available.\n";
-            return 0;
+            $msg .= "No configuration available in backend.\n";
         }
         my $r;
         unless ( ref( $self->{refLocalStorage} ) ) {
@@ -170,28 +173,48 @@ sub getConf {
                 $r = $self->getDBConf($args);
             }
         }
-        if ( $args->{clean} ) {
-            delete $r->{reVHosts};
-        }
-        else {
-            print STDERR
-              "Warning: key is not defined, set it in the manager !\n"
-              unless ( $r->{key} );
-            eval {
-                $r->{cipher} =
-                  Lemonldap::NG::Common::Crypto->new( $r->{key}
-                      || 'lemonldap-ng-key' );
-            };
-            if ($@) {
-                $msg .= "Bad key: $@. \n";
-                return $r;
+
+        # Get default values
+        my $confAttributes = Lemonldap::NG::Common::Conf::Attributes->new();
+
+        my @attributes = $confAttributes->meta()->get_attribute_list();
+
+        foreach my $name (@attributes) {
+            unless ( defined $r->{$name} ) {
+                $r->{$name} = $confAttributes->$name;
             }
         }
 
+        # Create cipher object
+        eval { $r->{cipher} = Lemonldap::NG::Common::Crypto->new( $r->{key} ); };
+        if ($@) {
+            $msg .= "Bad key: $@. \n";
+            return $r;
+        }
+
+        # Adapt some values
+
         # Convert old option useXForwardedForIP into trustedProxies
-        if ( $r->{useXForwardedForIP} == 1 ) {
+        if ( defined $r->{useXForwardedForIP}
+            and $r->{useXForwardedForIP} == 1 )
+        {
             $r->{trustedProxies} = '*';
         }
+
+        # Force Choice backend
+        if ( $r->{authentication} eq "Choice" ) {
+            $r->{userDB}     = "Choice";
+            $r->{passwordDB} = "Choice";
+        }
+
+        # Some parameters expect key name (example), not variable ($example)
+        foreach (qw/whatToTrace/) {
+            if ( defined $r->{$_} ) {
+                $r->{$_} =~ s/^\$//;
+            }
+        }
+
+        # Return configuration hash
         return $r;
     }
 }
@@ -208,7 +231,10 @@ sub getLocalConf {
     my $r = {};
 
     $section ||= DEFAULTSECTION;
-    $file ||= $self->{confFile} || DEFAULTCONFFILE;
+    $file ||=
+         $self->{confFile}
+      || $ENV{LLNG_DEFAULTCONFFILE}
+      || DEFAULTCONFFILE;
     $loaddefault = 1 unless ( defined $loaddefault );
     my $cfg;
 
@@ -311,12 +337,6 @@ sub getDBConf {
     }
     my $conf = $self->load( $args->{cfgNum} );
     $msg .= "Get configuration $conf->{cfgNum}.\n";
-    my $re = Regexp::Assemble->new();
-    foreach ( keys %{ $conf->{locationRules} } ) {
-        $_ = quotemeta($_);
-        $re->add($_);
-    }
-    $conf->{reVHosts} = $re->as_string;
     $self->setLocalConf($conf)
       if ( $self->{refLocalStorage} and not( $args->{noCache} ) );
     return $conf;
@@ -340,7 +360,8 @@ sub available {
 # Call lastCfg() from the $self->{type} package.
 # @return Number of the last configuration available
 sub lastCfg {
-    return &{ $_[0]->{type} . '::lastCfg' }(@_);
+    my $result = &{ $_[0]->{type} . '::lastCfg' }(@_) || "0";
+    return $result;
 }
 
 ## @method boolean lock()
@@ -415,7 +436,7 @@ Web-SSO configuration.
                   # To use local cache, set :
                   localStorage => "Cache::FileCache",
                   localStorageOptions = {
-                      'namespace' => 'lemonldap-ng',
+                      'namespace' => 'lemonldap-ng-config',
                       'default_expires_in' => 600,
                       'directory_umask' => '007',
                       'cache_root' => '/tmp',
@@ -515,7 +536,7 @@ L<http://lemonldap-ng.org/>
 
 =item Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
-=item Franois-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+=item François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
 =item Xavier Guimard, E<lt>x.guimard@free.frE<gt>
 
@@ -541,7 +562,7 @@ L<http://forge.objectweb.org/project/showfiles.php?group_id=274>
 
 =item Copyright (C) 2012 by Sandro Cazzaniga, E<lt>cazzaniga.sandro@gmail.comE<gt>
 
-=item Copyright (C) 2012 by Franois-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
+=item Copyright (C) 2012 by François-Xavier Deltombe, E<lt>fxdeltombe@gmail.com.E<gt>
 
 =item Copyright (C) 2009, 2010, 2011, 2012, 2013 by Clement Oudot, E<lt>clem.oudot@gmail.comE<gt>
 
