@@ -11,7 +11,7 @@ use Lemonldap::NG::Common::Conf::Constants;    #inherits
 use Lemonldap::NG::Common::Conf::Serializer;
 use Encode;
 
-our $VERSION = '1.2.4';
+our $VERSION = '1.4.0';
 
 BEGIN {
     *Lemonldap::NG::Common::Conf::ldap = \&ldap;
@@ -92,6 +92,11 @@ sub ldap {
         ( $self->{ldapPort} ? ( port => $self->{ldapPort} ) : () ),
     );
 
+    unless ($ldap) {
+        $Lemonldap::NG::Common::Conf::msg .= "$@\n";
+        return;
+    }
+
     # Start TLS if needed
     if ($useTls) {
         my %h = split( /[&=]/, $tlsParam );
@@ -168,7 +173,6 @@ sub store {
     }
 
     $self->ldap->unbind() && delete $self->{ldap};
-    $self->unlock;
     return $fields->{cfgNum};
 }
 
